# Copyright (C) 2017-2022 Chris N. Richardson and Garth N. Wells
#
# This file is part of FEniCS-miniapp (https://www.fenicsproject.org)
#
# SPDX-License-Identifier:    MIT

import basix.ufl
from ufl import (Coefficient, Identity, FunctionSpace, Mesh, TestFunction, TrialFunction,
                 dx, grad, inner, tetrahedron, tr)

# Elasticity parameters
E = 1.0e6
nu = 0.3
mu = E / (2.0 * (1.0 + nu))
lmbda = E * nu / ((1.0 + nu) * (1.0 - 2.0 * nu))
cell = tetrahedron

# Load namespace
ns = vars()

forms = []
for degree in range(1, 4):
    element = basix.ufl.element("Lagrange", "tetrahedron", degree, shape=(3, ))
    domain = Mesh(basix.ufl.element("Lagrange", "tetrahedron", 1, shape=(3, )))
    space = FunctionSpace(domain, element)

    u, v = TrialFunction(space), TestFunction(space)
    f = Coefficient(space)

    def eps(v):
        return 0.5*(grad(v) + grad(v).T)

    def sigma(v):
        return 2.0*mu*eps(v) + lmbda*tr(eps(v))*Identity(3)

    # Add forms to namespace with names a1, a2, a3 etc.
    aname = 'a' + str(degree)
    Lname = 'L' + str(degree)
    ns[aname] = inner(sigma(u), eps(v))*dx
    ns[Lname] = inner(f, v)*dx

    del u, v, f
    forms += [ns[aname], ns[Lname]]



# Copyright (C) 2017-2022 Chris N. Richardson and Garth N. Wells
#
# This file is part of FEniCS-miniapp (https://www.fenicsproject.org)
#
# SPDX-License-Identifier:    MIT

import basix.ufl
from ufl import (Coefficient, FunctionSpace, TestFunction, TrialFunction, Mesh, action, ds,
                 dx, grad, inner, tetrahedron)

# Load namespace
ns = vars()

forms = []
for degree in range(1, 4):
    element = basix.ufl.element("Lagrange", "tetrahedron", degree)
    domain = Mesh(basix.ufl.element("Lagrange", "tetrahedron", 1, shape=(3,)))
    space = FunctionSpace(domain, element)

    u = TrialFunction(space)
    v = TestFunction(space)
    f = Coefficient(space)
    g = Coefficient(space)
    un = Coefficient(space)

    aname = 'a' + str(degree)
    Lname = 'L' + str(degree)
    Mname = 'M' + str(degree)

    # Insert into namespace so that the forms will be named a1, a2, a3 etc.
    ns[aname] = inner(grad(u), grad(v))*dx
    ns[Lname] = f*v*dx + g*v*ds
    ns[Mname] = action(ns[aname], un)

    # Delete, so that the forms will get unnamed args and coefficients
    # and default to v_0, v_1, w0, w1 etc.
    del u, v, f, g, un

    forms += [ns[aname], ns[Lname], ns[Mname]]
